@@ -5,12 +5,13 @@ from .structs import (
 from .market import Exchange
 from .private import Account
 from .api import ApiError, ApiProvider
+from . import pairs, coins
 
 __all__ = [
-    'OrderSide', 'OrderStatus', 'OrderType', 'Pair', 'Coin',
+    'OrderSide', 'OrderStatus', 'OrderType', 'pairs', 'Pair', 'coins', 'Coin',
     'Period', 'Candle', 'MarketTrade', 'PrivateTrade',
     'Exchange', 'Account',
     'ApiError', 'ApiProvider'
 ]
 
-__version__ = '0.5.1'
+__version__ = '0.7'
